package cn.minbb.evaluationsystemserver.controller;

import cn.minbb.evaluationsystemserver.entity.*;
import cn.minbb.evaluationsystemserver.service.*;
import cn.minbb.evaluationsystemserver.util.ResponseMap;
import cn.minbb.evaluationsystemserver.util.Result;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Controller
@RequestMapping("")
public class ServerController {

    private final UserService userService;
    private final VersionService versionService;
    private final ClassificationService classificationService;
    private final QuestionService questionService;
    private final AnswerService answerService;
    private final PaperService paperService;

    @Autowired
    public ServerController(UserService userService,
                            VersionService versionService,
                            ClassificationService classificationService,
                            QuestionService questionService,
                            AnswerService answerService,
                            PaperService paperService) {
        this.userService = userService;
        this.versionService = versionService;
        this.classificationService = classificationService;
        this.questionService = questionService;
        this.answerService = answerService;
        this.paperService = paperService;
    }

    /**
     * 获取版本信息
     *
     * @return
     */
    @PostMapping(value = "/version", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String getVersion() {
        ResponseMap responseMap = new ResponseMap();
        responseMap.putResult(Result.SUCCESS);
        responseMap.putSummary("查找成功");
        responseMap.putData(versionService.getLatestVersion());
        return JSONObject.toJSONString(responseMap.getMap());
    }

    @GetMapping(value = "/download/apk/{versionCode}")
    public ResponseEntity<InputStreamResource> downloadAPK(@PathVariable("versionCode") Integer versionCode) throws IOException {
        Version version = versionService.findVersionByVersionCode(versionCode);
        if (version != null) {
            String apkPath = "C:\\Evaluation\\download\\apk\\" + version.getFileName();
            FileSystemResource file = new FileSystemResource(apkPath);
            HttpHeaders headers = new HttpHeaders();
            headers.add("Cache-Control", "no-cache, no-store, must-revalidate");
            headers.add("Content-Disposition", String.format("attachment; filename=\"%s\"", file.getFilename()));
            headers.add("Pragma", "no-cache");
            headers.add("Expires", "0");

            return ResponseEntity
                    .ok()
                    .headers(headers)
                    .contentLength(file.contentLength())
                    // .contentType(MediaType.parseMediaType("application/octet-stream"))
                    .contentType(MediaType.parseMediaType("application/x-shockwave-flash"))
                    .body(new InputStreamResource(file.getInputStream()));
        } else {
            return null;
        }
    }

    /**
     * 获取题目类别
     *
     * @return
     */
    @PostMapping(value = "/classification", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String getClassification() {
        ResponseMap responseMap = new ResponseMap();
        responseMap.putResult(Result.SUCCESS);
        responseMap.putSummary("查找成功");
        List<cn.minbb.evaluationsystemserver.entity.Classification> classificationList = classificationService.findEnabledClassification();
        for (cn.minbb.evaluationsystemserver.entity.Classification classification : classificationList) {
            classification.setQuestionList(null);
        }
        responseMap.putData(classificationList);
        return JSONObject.toJSONString(responseMap.getMap());
    }

    /**
     * 随机抽取试题
     *
     * @param classification_id 试题类别 e_id
     * @return
     */
    @PostMapping(value = "/question", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String getQuestion(@RequestParam(value = "id") Long classification_id, @RequestParam(value = "user") String userJSON) {
        ResponseMap responseMap = new ResponseMap();
        if (!userJSON.isEmpty()) {
            try {
                User userClient = JSONObject.parseObject(userJSON, User.class);
                User userServer = userService.findUserByUsername(userClient.getUsername());
                if (userServer != null && userServer.getPassword().equals(userClient.getPassword())) {
                    if (classification_id != null) {
                        float paperScore = 0;
                        List<Classification> classificationList = new ArrayList<>();
                        classificationList.add(classificationService.findByE_Id(classification_id));
                        List<Question> questionList = questionService.randomSelection(classificationList, 20);
                        // 计算总分
                        for (Question question : questionList) {
                            paperScore += question.getScore();
                        }
                        if (questionList.size() != 0) {
                            // 保存数据
                            Paper paper = paperService.save(new Paper(userServer, paperScore, questionList));
                            // 剔除敏感数据
                            paper.setUser(null);
                            for (Question question : paper.getQuestionList()) {
                                question.setClassificationList(null);
                                question.setPaperList(null);
                            }
                            responseMap.putResult(Result.SUCCESS);
                            responseMap.putSummary("抽取成功");
                            responseMap.putData(paper);
                        } else {
                            responseMap.putResult(Result.FAILED);
                            responseMap.putSummary("抱歉，题库中没有可抽取的题目！");
                            responseMap.putData(null);
                        }
                    } else {
                        responseMap.putResult(Result.FAILED);
                        responseMap.putSummary("抽取失败，试题 ID 不能为空");
                        responseMap.putData(null);
                    }
                } else {
                    responseMap.putResult(Result.FAILED);
                    responseMap.putSummary("身份验证失败");
                }
            } catch (Exception e) {
                e.printStackTrace();
                responseMap.putResult(Result.FAILED);
                responseMap.putSummary("请提供正确格式的用户信息");
            }
        } else {
            responseMap.putResult(Result.FAILED);
            responseMap.putSummary("用户信息不存在");
        }
        System.out.println("发送数据量 = " + JSONObject.toJSONString(responseMap.getMap()).length());
        System.out.println("发送数据 = " + JSONObject.toJSONString(responseMap.getMap()));
        return JSONObject.toJSONString(responseMap.getMap());
    }

    /**
     * 在线判题
     *
     * @param paperJSON 试卷
     * @param userJSON  用户
     * @return
     */
    @PostMapping(value = "/question/evaluate", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String evaluateQuestion(@RequestParam(value = "paper") String paperJSON, @RequestParam(value = "user") String userJSON) {
        System.out.println("paper = " + paperJSON);
        System.out.println("user = " + userJSON);
        ResponseMap responseMap = new ResponseMap();
        if (!paperJSON.isEmpty() && !userJSON.isEmpty()) {
            try {
                Paper paperClient = JSONObject.parseObject(paperJSON, Paper.class);
                User userClient = JSONObject.parseObject(userJSON, User.class);
                User userServer = userService.findUserByUsername(userClient.getUsername());
                if (userServer != null && userServer.getPassword().equals(userClient.getPassword())) {
                    Paper paperServer = paperService.findPaperByE_Id(paperClient.getE_id());
                    if (paperServer.getUser().getE_id().equals(userServer.getE_id())) {
                        if (paperServer.getMark() == null) {
                            float mark = 0;
                            List<Question> questionListServer = paperServer.getQuestionList();
                            List<Question> questionListClient = paperClient.getQuestionList();
                            for (Question questionServer : questionListServer) {
                                for (Question questionClient : questionListClient) {
                                    if (questionServer.getE_id().equals(questionClient.getE_id())) {
                                        // 获取用户答案集合
                                        Set<Answer> answerSet = questionClient.getAnswerSet();
                                        if (answerSet != null && answerSet.size() == 1) {
                                            // 查询答案集合中第一个答案的分值
                                            mark += answerService.findAnswerByE_Id(answerSet.toArray(new Answer[1])[0].getE_id()).getScore();
                                        } else {
                                            // 题目未做
                                            mark += 0;
                                        }
                                        break;
                                    }
                                }
                            }
                            Paper paper = paperService.submit(paperServer, mark);
                            paper.setUser(null);
                            for (Question question : paper.getQuestionList()) {
                                question.setAnswerSet(null);
                                question.setPaperList(null);
                                question.setClassificationList(null);
                            }
                            responseMap.putResult(Result.SUCCESS);
                            responseMap.putSummary("成绩 = " + mark);
                            responseMap.putData(paper);
                        } else {
                            responseMap.putResult(Result.FAILED);
                            responseMap.putSummary("试卷已提交并评测过，分数值 = " + paperServer.getMark());
                        }
                    } else {
                        responseMap.putResult(Result.FAILED);
                        responseMap.putSummary("试卷非本人");
                    }
                } else {
                    responseMap.putResult(Result.FAILED);
                    responseMap.putSummary("身份验证失败");
                }
            } catch (Exception e) {
                e.printStackTrace();
                responseMap.putResult(Result.FAILED);
                responseMap.putSummary("请提供正确格式的试卷和用户信息");
            }
        } else {
            responseMap.putResult(Result.FAILED);
            responseMap.putSummary("请提供正确格式的试卷和用户信息");
        }
        return JSONObject.toJSONString(responseMap.getMap());
    }

    @PostMapping(value = "/papers", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String historyPapers(@RequestParam(value = "user") String userJSON, @RequestParam(value = "page") int page, @RequestParam(value = "size") int size) {
        ResponseMap responseMap = new ResponseMap();
        if (!userJSON.isEmpty()) {
            try {
                User userClient = JSONObject.parseObject(userJSON, User.class);
                User userServer = userService.findUserByUsername(userClient.getUsername());
                if (userServer != null && userServer.getPassword().equals(userClient.getPassword())) {
                    if (page >= 0 && size >= 0) {
                        List<Paper> paperList = paperService.findHistoryPapers(userServer, page, size);
                        for (Paper paper : paperList) {
                            paper.setUser(null);
                            for (Question question : paper.getQuestionList()) {
                                question.setAnswerSet(null);
                                question.setPaperList(null);
                                question.setClassificationList(null);
                            }
                        }
                        responseMap.putResult(Result.SUCCESS);
                        responseMap.putSummary("数据条数 = " + paperList.size());
                        responseMap.putData(paperList);
                    } else {
                        responseMap.putResult(Result.FAILED);
                        responseMap.putSummary("索引页不能小于零");
                    }
                } else {
                    responseMap.putResult(Result.FAILED);
                    responseMap.putSummary("身份验证失败");
                }
            } catch (Exception e) {
                responseMap.putResult(Result.FAILED);
                responseMap.putSummary("请提供正确格式的用户信息");
                e.printStackTrace();
            }
        } else {
            responseMap.putResult(Result.FAILED);
            responseMap.putSummary("请提供正确格式的用户信息");
        }
        return JSONObject.toJSONString(responseMap.getMap());
    }
}
