package cn.minbb.evaluationsystemserver.entity;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import javax.persistence.Version;
import java.util.Date;
import java.util.List;
import java.util.Set;

@Entity
@Table(name = "user")
@NoArgsConstructor
public class User {

    @Getter
    @Setter
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "e_id", nullable = false, columnDefinition = "BIGINT COMMENT '用户实体ID'")
    // 实体 Id，主键，自增
    private Long e_id;

    @Getter
    @Setter
    @Column(name = "username", nullable = false, columnDefinition = "VARCHAR(64) COMMENT '用户名'")
    private String username;

    @Getter
    @Setter
    @Column(name = "password", nullable = false, columnDefinition = "VARCHAR(32) COMMENT '密码'")
    private String password;

    @Getter
    @Setter
    @Column(name = "name", length = 64, columnDefinition = "VARCHAR(64) COMMENT '用户姓名'")
    private String name;

    @Getter
    @Setter
    @Column(name = "email", columnDefinition = "VARCHAR(255) COMMENT '邮箱'")
    private String email;

    @Getter
    @Setter
    @Column(name = "phone", length = 16, columnDefinition = "VARCHAR(16) COMMENT '电话'")
    private String phone;

    @Getter
    @Setter
    @Column(name = "qq", length = 16, columnDefinition = "VARCHAR(16) COMMENT 'QQ'")
    private String qq;

    @Getter
    @Setter
    @Column(name = "avatar", columnDefinition = "VARCHAR(255) COMMENT '头像'")
    private String avatar;

    @Getter
    @Setter
    @Column(name = "permission", nullable = false, columnDefinition = "TINYINT DEFAULT 1 COMMENT '权限'")
    private Integer permission;

    /**
     * 在 OneToMany 和 ManyToMany 中，只要是加载多的一方，那么它的延迟属性的默认值就是延迟加载，即 fetch = FetchType.LAZY。共同点是后面都是 ToMany
     * 在 ManyToOne 和 OneToOne 中，只要是加载一的一方，那么它的延迟属性的默认值就是立即记载，即 fetch = FetchType.EAGER。共同点是后面都是 ToOne
     * <p>
     * CascadeType.REFRESH --> 级联刷新
     * CascadeType.PERSIST --> 级联持久化，或者叫做级联保存。当保存对象时，同时保存该对象的子对象
     * CascadeType.MERGE ----> 级联合并，也可以叫做级联更新。当更新对象时，同时更新该对象的子对象
     * CascadeType.REMOVE ---> 级联删除。。。。。。。。。。。当删除对象时，同时删除该对象的子对象
     * 注意：CascadeType.REFRESH 只适用于实体管理器的 refresh() 方法
     * CascadeType.PERSIST 只适用于实体管理器的 persist() 方法
     * CascadeType.MERGE 只适用于实体管理器的 merge() 方法
     * CascadeType.REMOVE 只适用于实体管理器的 remove() 方法，并且对于 JPQL 中的 delete from... 是不会起作用的
     * <p>
     * 小结：也就是说这里的四个级联类型，是与实体管理器 EntityManager 的四个方法一一对应的，而与 JPQL 无关
     * 在 JPA 规范中，如果是一对多或者多对一关系的话，那么多的一方为关系维护端
     * 关系维护端负责外键记录的更新，关系被维护端是没有权利更新外键字段的
     * 这里 mappedBy 属性，等于是声明了该类为关系的被维护端
     * mappedBy 的值指定的就是关系维护端中维护该关系的属性，类似于 hibernate 中的 inverse = "true" 属性
     * 这里 mappedBy = "user" 里面的 user 对应的是 Paper.java 中的 user 属性
     */
    @Getter
    @Setter
    @OneToMany(cascade = {CascadeType.REFRESH}, mappedBy = "user", fetch = FetchType.LAZY)
    // 试卷（列表）
    private List<Paper> paperList;

    @Getter
    @Setter
    @OneToMany(cascade = {CascadeType.REFRESH}, mappedBy = "user", fetch = FetchType.LAZY)
    // 反馈（列表）
    private List<Feedback> feedbackList;

    @Getter
    @Setter
    @OneToMany(cascade = {CascadeType.REFRESH}, mappedBy = "userFrom", fetch = FetchType.LAZY)
    // 消息发送者（列表）
    private List<Message> messageListFrom;

    @Getter
    @Setter
    @ManyToMany(cascade = CascadeType.REFRESH)
    @JoinTable(name = "user_message",
            joinColumns = {@JoinColumn(name = "user_id", referencedColumnName = "e_id", columnDefinition = "BIGINT COMMENT '消息接收者用户实体ID'")},
            inverseJoinColumns = {@JoinColumn(name = "message_id", referencedColumnName = "e_id", columnDefinition = "BIGINT COMMENT '消息实体ID'")})
    // 消息接收者（集合）
    private Set<Message> messageSetTo;

    @Getter
    @Setter
    @OneToMany(cascade = {CascadeType.REFRESH}, mappedBy = "user", fetch = FetchType.LAZY)
    // 试卷（列表）
    private List<Logs> logsList;

    @Getter
    @Setter
    @Column(name = "created_at", columnDefinition = "DATETIME COMMENT '创建时间'")
    @CreationTimestamp
    private Date createdAt;

    @Getter
    @Setter
    @Column(name = "updated_at", columnDefinition = "DATETIME COMMENT '更新时间'")
    @UpdateTimestamp
    private Date updatedAt;

    @Version
    @Column(name = "version", columnDefinition = "BIGINT COMMENT '版本号'")
    public Long version;

    public User(String username, String password) {
        this.username = username;
        this.password = password;
    }
}
