package cn.minbb.evaluationsystemserver.websocket;

import cn.minbb.evaluationsystemserver.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.messaging.simp.config.MessageBrokerRegistry;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.WebSocketMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.config.annotation.AbstractWebSocketMessageBrokerConfigurer;
import org.springframework.web.socket.config.annotation.EnableWebSocketMessageBroker;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketTransportRegistration;
import org.springframework.web.socket.handler.WebSocketHandlerDecorator;
import org.springframework.web.socket.handler.WebSocketHandlerDecoratorFactory;

/**
 * WebSocket 配置
 */
@Configuration
// 开启使用 STOMP 协议来传输基于代理的消息，Broker 就是代理的意思
@EnableWebSocketMessageBroker
public class WebSocketConfig extends AbstractWebSocketMessageBrokerConfigurer {

    private UserService userService;

    @Autowired
    public WebSocketConfig(UserService userService) {
        this.userService = userService;
    }

    // 注册 STOMP 协议的节点，并指定映射的 URL
    @Override
    public void registerStompEndpoints(StompEndpointRegistry stompEndpointRegistry) {
        // 注册 STOMP 协议节点，允许使用 socketJs 方式，同时指定使用 SockJS 协议，访问点为 ws，允许跨域
        stompEndpointRegistry.addEndpoint("/ws").setAllowedOrigins("*").addInterceptors(new WebSocketHandshakeInterceptor()).withSockJS();
    }

    // 配置消息代理
    @Override
    public void configureMessageBroker(MessageBrokerRegistry messageBrokerRegistry) {
        messageBrokerRegistry.setApplicationDestinationPrefixes("/app");          // 设置客户端订阅消息的基础路径，全局使用的消息前缀（客户端订阅路径上会体现出来）
        messageBrokerRegistry.enableSimpleBroker("/queue", "/topic");     // 设置服务器广播消息的基础路径
        messageBrokerRegistry.setUserDestinationPrefix("/user/");       // 点对点使用的订阅前缀（客户端订阅路径上会体现出来），不设置的话，默认也是/user/
        // messageBrokerRegistry.setPathMatcher(new AntPathMatcher("."));         //可以已“.”来分割路径，看看类级别的 @messageMapping 和方法级别的 @messageMapping
    }

    @Override
    public void configureWebSocketTransport(WebSocketTransportRegistration registration) {
        registration.addDecoratorFactory(new WebSocketHandlerDecoratorFactory() {
            @Override
            public WebSocketHandler decorate(WebSocketHandler handler) {
                return new WebSocketHandlerDecorator(handler) {
                    @Override
                    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
                        super.afterConnectionEstablished(session);
                    }

                    @Override
                    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
                        super.afterConnectionClosed(session, closeStatus);
                    }

                    @Override
                    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
                        super.handleMessage(session, message);
                    }

                    @Override
                    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
                        super.handleTransportError(session, exception);
                    }
                };
            }
        });
        super.configureWebSocketTransport(registration);
    }
}
